//
//  ViewController.swift
//  Citizen LayoutSDK Sample
//
//  Created by Yoshikazu Noguchi on 2017/03/15.
//  Copyright © 2017年 CITIZEN SYSTEMS JAPAN CO., LTD. All rights reserved.
//

import UIKit
import CSJPOSLibSwift
import PrintEngine

class ViewController: UIViewController, UITextFieldDelegate {

    @IBOutlet weak var textLayoutData: UITextField!
    @IBOutlet weak var textHeaderData: UITextField!
    @IBOutlet weak var textItem01Data: UITextField!
    @IBOutlet weak var textItem02Data: UITextField!
    @IBOutlet weak var textItem03Data: UITextField!
    @IBOutlet weak var textItem04Data: UITextField!
    @IBOutlet weak var textItem05Data: UITextField!
    @IBOutlet weak var textFooterData: UITextField!
    @IBOutlet weak var textIPAddressData: UITextField!
    @IBOutlet weak var textBDAddressData: UITextField!
    @IBOutlet weak var segmentInterfaceType: UISegmentedControl!
    @IBOutlet weak var switchUpsideDown: UISwitch!
    @IBOutlet weak var switchReplaceLogo: UISwitch!

    var listLayoutData: [String] = []
    var textActiveField: UITextField!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        
        self.textItem01Data.delegate = self
        self.textItem01Data.keyboardType = UIKeyboardType.asciiCapable
        self.textItem01Data.returnKeyType = UIReturnKeyType.done
        self.textItem02Data.delegate = self
        self.textItem02Data.keyboardType = UIKeyboardType.asciiCapable
        self.textItem02Data.returnKeyType = UIReturnKeyType.done
        self.textItem03Data.delegate = self
        self.textItem03Data.keyboardType = UIKeyboardType.asciiCapable
        self.textItem03Data.returnKeyType = UIReturnKeyType.done
        self.textItem04Data.delegate = self
        self.textItem04Data.keyboardType = UIKeyboardType.asciiCapable
        self.textItem04Data.returnKeyType = UIReturnKeyType.done
        self.textItem05Data.delegate = self
        self.textItem05Data.keyboardType = UIKeyboardType.asciiCapable
        self.textItem05Data.returnKeyType = UIReturnKeyType.done
        self.textHeaderData.delegate = self
        self.textHeaderData.keyboardType = UIKeyboardType.asciiCapable
        self.textHeaderData.returnKeyType = UIReturnKeyType.done
        self.textFooterData.delegate = self
        self.textFooterData.keyboardType = UIKeyboardType.URL
        self.textFooterData.returnKeyType = UIReturnKeyType.done
        self.textIPAddressData.delegate = self
        self.textIPAddressData.keyboardType = UIKeyboardType.numbersAndPunctuation
        self.textIPAddressData.returnKeyType = UIReturnKeyType.done
        self.textBDAddressData.delegate = self
        self.textBDAddressData.keyboardType = UIKeyboardType.asciiCapable
        self.textBDAddressData.returnKeyType = UIReturnKeyType.done
        
        let folderPath = Bundle.main.path(forAuxiliaryExecutable: "LayoutData")!
        let manager = FileManager.default
        self.listLayoutData = try! manager.contentsOfDirectory(atPath: folderPath)
        
        self.switchUpsideDown.isOn = false
        self.switchReplaceLogo.isOn = false
        self.segmentInterfaceType.selectedSegmentIndex = 0
        
        self.textHeaderData.text = "CITIZEN"
        self.textFooterData.text = "http://www.citizen-systems.co.jp"
        self.textItem01Data.text = "Sample ABC"
        self.textItem02Data.text = "Sample 123"
        self.textItem03Data.text = "Sample 456"
        self.textItem04Data.text = "Sample 789"
        self.textItem05Data.text = "Coupon"
        self.textIPAddressData.text = "192.168.0.21"
        //self.textIPAddressData.text = "192.168.129.60"
        self.textBDAddressData.text = "00:01:90:DF:C3:86"
        self.textLayoutData.text = self.listLayoutData[0]
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        self.configureObserver()
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        
        super.viewWillDisappear(animated)
        self.removeObserver()
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.resignFirstResponder()
        return true
    }
    
    func textFieldShouldBeginEditing(_ textField: UITextField) -> Bool {
        textActiveField = textField
        return true
    }

    func textFieldShouldEndEditing(_ textField: UITextField) -> Bool {
        textActiveField = nil
        return true
    }
    
    func configureObserver() {
        let notification = NotificationCenter.default
        notification.addObserver(self, selector: #selector(keyboardWillShow(notification:)), name: UIResponder.keyboardWillShowNotification, object: nil)
        notification.addObserver(self, selector: #selector(keyboardWillHide(notification:)), name: UIResponder.keyboardWillHideNotification, object: nil)
    }
    
    func removeObserver() {
        let notification = NotificationCenter.default
        notification.removeObserver(self)
    }
    
    @objc func keyboardWillShow(notification: Notification?) {
        if nil != textActiveField {
            let textFieldRect = textActiveField.frame
            let rect = (notification?.userInfo?[UIResponder.keyboardFrameEndUserInfoKey] as? NSValue)?.cgRectValue
            if textFieldRect.maxY < rect!.minY{
                return
            }
            let duration: TimeInterval? = notification?.userInfo?[UIResponder.keyboardAnimationDurationUserInfoKey] as? Double
            UIView.animate(withDuration: duration!, animations: { () in
                let transform = CGAffineTransform(translationX: 0, y: -((textFieldRect.maxY - rect!.origin.y) + 5))
                self.view.transform = transform
            })
        } else {
            self.view.endEditing(true)
        }
    }
    
    @objc func keyboardWillHide(notification: Notification?) {
        let duration: TimeInterval? = notification?.userInfo?[UIResponder.keyboardAnimationCurveUserInfoKey] as? Double
        UIView.animate(withDuration: duration!, animations: { () in
            let transform = CGAffineTransform(translationX: 0, y: 0)
            self.view.transform = transform
        })
    }
    
    func showLayoutData() {
        let alert = UIAlertController(title: "Select Layout Data", message: "", preferredStyle: .alert)
        
        for index in (0 ..< self.listLayoutData.count) {
            let layoutFile = UIAlertAction(title: self.listLayoutData[index], style: .destructive, handler:{
                (action: UIAlertAction!) -> Void in
                self.textLayoutData.text = self.listLayoutData[index]
            })
            alert.addAction(layoutFile)
        }
        self.present(alert, animated: true, completion: nil)
    }
    
    @IBAction func touchDownLayoutData(_ sender: Any) {
        self.showLayoutData()
    }
    
    func doPrint(connectType: Int32, addressValue: String, lpe: PrintEngine, isUpsideDown: Bool) -> Int32 {
        
        var result = CMP_SUCCESS
        
        // ESCPOSPrinter Class
        let printer: ESCPOSPrinter? = CSJPOSLibSwift.ESCPOSPrinter()
        
        // Character set
        result = printer!.setEncoding(String.Encoding.isoLatin1)    // Latin-1
        //result = printer!.setEncoding(String.Encoding.shiftJIS)   // Japanese
        
        // Connect to the printer
        result = printer!.connect(connectType, withAddrress: addressValue)
        if CMP_SUCCESS == result {
            
            // Output the print data
            _ = lpe.doPrint(printer: printer!, isReverse: isUpsideDown)
            
            // Disconnect the printer
            result = printer!.disconnect()
        
        } else {
            
            // Connect error
            let errorMessage = "ESCPOSPrinter connect() error result : \(String (format: "%d", result))"
            let alert = UIAlertController(title: "Error", message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default))
            self.present(alert, animated: true, completion: nil)
        }
        return result
    }
    
    func doPrint(connectType: Int32, addressValue: String, layoutdataPath: String, isUpsideDown: Bool, isReplaceLogo: Bool, header: String, footer: String, items: [String]) {
        
        let stream: InputStream? = InputStream(fileAtPath: layoutdataPath)
        
        // LayoutPrintEngine Class
        let lpe: PrintEngine? = PrintEngine()
        var result: Int32
        result = lpe!.open(stream: stream!)
        if 0 == result {
            result = lpe!.beginPrint()
            var frameIndex: Int32
            
            // Header frame
            frameIndex = lpe!.initFrame(frameName: "Frame1")
            result = lpe!.setPartsData(frameIndex: frameIndex, partsName: "Text1", setText: header)
            
            if isReplaceLogo {
                let bundlePath = Bundle.main.path(forResource: "resources", ofType: "bundle")!
                let budle = Bundle(path: bundlePath)
                let logoPath = budle?.path(forResource: "newlogo", ofType: "png")
                let fileHandle = FileHandle(forReadingAtPath: logoPath!)
                let imageData = FileHandle.readDataToEndOfFile(fileHandle!)
                let replaceImage = UIImage(data: imageData())
                result = lpe!.setPartsData(frameIndex: frameIndex, partsName: "Image1", image: replaceImage!)
            }
            result = lpe!.addFrame(frameIndex: frameIndex)
            
            // Items frame
            var i = 7
            for (_, element) in items.enumerated() {
                frameIndex = lpe!.initFrame(frameName: "Frame2")
                result = lpe!.setPartsData(frameIndex: frameIndex, partsName: "Text1", setText: element)
                let price1 = "99999999.99"
                let start = i
                let end = price1.count
                //let end = price1.characters.count
                var price2 = price1[price1.index(price1.startIndex, offsetBy: start)..<price1.index(price1.startIndex, offsetBy: end)]
                //var price2 = price1.substring(with: price1.index(price1.startIndex, offsetBy: start)..<price1.index(price1.startIndex, offsetBy: end))
                var arryPrice2 = price2.components(separatedBy: ".")
                let intPrice2_0 = Int(arryPrice2[0])
                let numPrice2_0 = NSNumber(value: intPrice2_0!)
                let formatter = NumberFormatter()
                formatter.numberStyle = NumberFormatter.Style.decimal
                formatter.groupingSeparator = ","
                formatter.groupingSize = 3
                arryPrice2[0] = formatter.string(from: numPrice2_0)!
                price2 = arryPrice2[0] + "." + arryPrice2[1]
                i -= 1
                result = lpe!.setPartsData(frameIndex: frameIndex, partsName: "Text2", setText: String(price2))
                //result = lpe!.setPartsData(frameIndex: frameIndex, partsName: "Text2", setText: price2)
                result = lpe!.addFrame(frameIndex: frameIndex)
            }
            
            // Footer frame
            frameIndex = lpe!.initFrame(frameName: "Frame3")
            result = lpe!.setPartsData(frameIndex: frameIndex, partsName: "Text1", setText: footer)
            result = lpe!.setPartsData(frameIndex: frameIndex, partsName: "Barcode1", setText: footer)
            result = lpe!.addFrame(frameIndex: frameIndex)
            
            // Coupon frame, Please do not execute addFrame method, using initFrame method only.
            frameIndex = lpe!.initFrame(frameName: "Frame4")
            
            // Output to the printer
            result = self.doPrint(connectType: connectType, addressValue: addressValue, lpe: lpe!, isUpsideDown: isUpsideDown)
            
            lpe!.endPrint()
            lpe!.close()
            
        } else {
            
            // PrintEngine open error
            let errorMessage = "PrintEngine open() error result : \(String (format: "%d", result))"
            let alert = UIAlertController(title: "Error", message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default))
            self.present(alert, animated: true, completion: nil)
        }
        
        stream!.close()
    }
    
    func getSelectedConnectType() -> Int32 {
        
        var connectType = CMP_PORT_WiFi
        let idx = segmentInterfaceType.selectedSegmentIndex
        
        if 0 == idx {
            connectType = CMP_PORT_WiFi
        } else if 1 == idx {
            connectType = CMP_PORT_BLUETOOTH
        } else if 2 == idx {
            connectType = CMP_PORT_USB
        } else {
            connectType = -1
        }
        return connectType
    }
    
    func getSelectedAddress() -> String {
        
        var address = String()
        let idx = segmentInterfaceType.selectedSegmentIndex
        
        if 0 == idx {
            address = self.textIPAddressData.text!
        } else if 1 == idx {
            address = self.textBDAddressData.text!
        } else {
            address = String();
        }
        return address
    }
    
    @IBAction func touchUpInsidePrint(_ sender: Any) {
        
        // Get the port type
        let connectType = self.getSelectedConnectType()
        
        // Get the address
        let address = self.getSelectedAddress()
        
        // Get the layout file
        var layoutdatapath = Bundle.main.path(forAuxiliaryExecutable: "LayoutData")!
        layoutdatapath = layoutdatapath + "/" + self.textLayoutData.text!
        
        // Get the upside-down setting
        let isUpsideDown = self.switchUpsideDown.isOn
        
        // Get the replace-logo setting
        let isReplaceLogo = self.switchReplaceLogo.isOn
        
        // Get the print data
        let header = self.textHeaderData.text!
        let footer = self.textFooterData.text!
        var items: [String] = []
        items.append(self.textItem01Data.text!)
        items.append(self.textItem02Data.text!)
        items.append(self.textItem03Data.text!)
        items.append(self.textItem04Data.text!)
        items.append(self.textItem05Data.text!)
        
        self.doPrint(connectType: connectType, addressValue: address, layoutdataPath: layoutdatapath, isUpsideDown: isUpsideDown, isReplaceLogo: isReplaceLogo, header: header, footer: footer, items: items)
    }
}

